@@ -41,9 +41,10 @@ extern "C" {
 #include "nmm.h"
 #include "nie.h"
 #include "nfo.h"
-#include <nco.h>
+#include "nco.h"
 
 #include <Soprano/Vocabulary/NAO>
+#include <Soprano/Vocabulary/RDF>
 
 #include <KDebug>
 #include <QtCore/QDateTime>
@@ -63,10 +64,13 @@ QStringList FFmpegExtractor::mimetypes()
 {
     QStringList types;
 
-    types << QLatin1String("video/mpeg");
+    types << QLatin1String("video/x-ms-asf");
     types << QLatin1String("video/x-msvideo");
-
-    //FIXME: Discover more types
+    types << QLatin1String("video/x-flv");
+    types << QLatin1String("video/quicktime");
+    types << QLatin1String("video/mpeg");
+    types << QLatin1String("video/x-ms-wmv");
+    types << QLatin1String("video/mp4");
 
     return types;
 }
@@ -203,6 +207,10 @@ SimpleResourceGraph FFmpegExtractor::extract(const QUrl& resUri, const QUrl& fil
 
         fileRes.addProperty( NMM::musicAlbum(), album );
         fileRes.addProperty( NAO::hasSubResource(), album );
+
+        // Just in case
+        if( !fileRes.contains(RDF::type(), NMM::MusicPiece()) )
+            fileRes.addType( NMM::MusicPiece() );
     }
 
     entry = av_dict_get(dict, "genre", NULL, 0);
@@ -238,4 +246,4 @@ SimpleResourceGraph FFmpegExtractor::extract(const QUrl& resUri, const QUrl& fil
 
 }
 
-NEPOMUK_EXPORT_EXTRACTOR( Nepomuk2::FFmpegExtractor, "nepomukffmpegextractor" )
\ No newline at end of file
+NEPOMUK_EXPORT_EXTRACTOR( Nepomuk2::FFmpegExtractor, "nepomukffmpegextractor" )
