@@ -39,6 +39,7 @@
 #include <KUrl>
 #include <KStandardDirs>
 #include <KConfigGroup>
+#include <KLocale>
 
 
 Nepomuk2::IndexScheduler::IndexScheduler( QObject* parent )
@@ -72,6 +73,16 @@ Nepomuk2::IndexScheduler::IndexScheduler( QObject* parent )
     connect( m_fileIQ, SIGNAL(startedIndexing()), this, SLOT(slotStartedIndexing()) );
     connect( m_fileIQ, SIGNAL(finishedIndexing()), this, SLOT(slotFinishedIndexing()) );
 
+    // Status String
+    connect( m_basicIQ, SIGNAL(beginIndexingFile(QUrl)), this, SIGNAL(statusStringChanged()) );
+    connect( m_basicIQ, SIGNAL(endIndexingFile(QUrl)), this, SIGNAL(statusStringChanged()) );
+    connect( m_basicIQ, SIGNAL(startedIndexing()), this, SIGNAL(statusStringChanged()) );
+    connect( m_basicIQ, SIGNAL(finishedIndexing()), this, SIGNAL(statusStringChanged()) );
+    connect( m_fileIQ, SIGNAL(beginIndexingFile(QUrl)), this, SIGNAL(statusStringChanged()) );
+    connect( m_fileIQ, SIGNAL(endIndexingFile(QUrl)), this, SIGNAL(statusStringChanged()) );
+    connect( m_fileIQ, SIGNAL(startedIndexing()), this, SIGNAL(statusStringChanged()) );
+    connect( m_fileIQ, SIGNAL(finishedIndexing()), this, SIGNAL(statusStringChanged()) );
+
     m_eventMonitor = new EventMonitor( this );
     connect( m_eventMonitor, SIGNAL(diskSpaceStatusChanged(bool)),
              this, SLOT(slotScheduleIndexing()) );
@@ -154,16 +165,6 @@ bool Nepomuk2::IndexScheduler::isIndexing() const
     return m_indexing;
 }
 
-QString Nepomuk2::IndexScheduler::currentFolder() const
-{
-    return KUrl(currentUrl()).directory();
-}
-
-QString Nepomuk2::IndexScheduler::currentFile() const
-{
-    return currentUrl().toLocalFile();
-}
-
 QUrl Nepomuk2::IndexScheduler::currentUrl() const
 {
     if( !m_fileIQ->currentUrl().isEmpty() )
@@ -203,6 +204,13 @@ void Nepomuk2::IndexScheduler::slotFinishedIndexing()
 void Nepomuk2::IndexScheduler::slotCleaningDone()
 {
     m_cleaner = 0;
+
+    for( int i=0; i<m_foldersToQueue.size(); i++ ) {
+        QPair< QString, UpdateDirFlags > pair = m_foldersToQueue[i];
+        m_basicIQ->enqueue( pair.first, pair.second );
+    }
+
+    m_fileIQ->start();
 }
 
 void Nepomuk2::IndexScheduler::updateDir( const QString& path, UpdateDirFlags flags )
@@ -226,17 +234,19 @@ void Nepomuk2::IndexScheduler::queueAllFoldersForUpdate( bool forceUpdate )
         flags |= ForceUpdate;
 
     // update everything again in case the folders changed
+    // If the cleaner is running, then we do not add the folders, instead they will be added
+    // in slotCleaningDone
     foreach( const QString& f, FileIndexerConfig::self()->includeFolders() ) {
-        m_basicIQ->enqueue( f, flags );
+        if( m_cleaner )
+            m_foldersToQueue.append( qMakePair( f, flags ) );
+        else
+            m_basicIQ->enqueue( f, flags );
     }
 }
 
 
 void Nepomuk2::IndexScheduler::slotConfigChanged()
 {
-    // TODO: only update folders that were added in the config
-    updateAll();
-
     if( m_cleaner ) {
         m_cleaner->kill();
         delete m_cleaner;
@@ -246,6 +256,14 @@ void Nepomuk2::IndexScheduler::slotConfigChanged()
     m_cleaner = new IndexCleaner( this );
     connect( m_cleaner, SIGNAL(finished(KJob*)), this, SLOT(slotCleaningDone()) );
     m_cleaner->start();
+
+    m_basicIQ->clear();
+    m_fileIQ->clear();
+
+    // Make sure this is called after the cleaner has been created, otherwise
+    // both the tasks will happen in parallel
+    // TODO: only update folders that were added in the config
+    updateAll();
 }
 
 
@@ -318,6 +336,38 @@ void Nepomuk2::IndexScheduler::slotScheduleIndexing()
     }
 }
 
+QString Nepomuk2::IndexScheduler::userStatusString() const
+{
+    bool indexing = isIndexing();
+    bool suspended = isSuspended();
+    bool cleaning = ( m_cleaner != 0 );
+    bool processing = !m_basicIQ->isEmpty();
+
+    if ( suspended ) {
+        return i18nc( "@info:status", "File indexer is suspended." );
+    }
+    else if ( cleaning ) {
+        // TODO: Change this string for 4.11
+        return i18nc( "@info:status", "Scanning for recent changes in files for desktop search");
+    }
+    else if ( indexing ) {
+        QUrl url = currentUrl();
+
+        if( url.isEmpty() ) {
+            return i18nc( "@info:status", "Indexing files for desktop search." );
+        }
+        else {
+            return i18nc( "@info:status", "Indexing %1", url.toLocalFile() );
+        }
+    }
+    else if ( processing ) {
+        return i18nc( "@info:status", "Scanning for recent changes in files for desktop search");
+    }
+    else {
+        return i18nc( "@info:status", "File indexer is idle." );
+    }
+}
+
 
 
 
