@@ -83,17 +83,6 @@ namespace {
     {
         Q_UNUSED( flags );
 
-        // Some symlink handling.
-        QFileInfo info(path);
-        // Check whether this path is a symlink to a path which has already been watched.
-        // If so, do not watch it. This does not solve *all* double-watching;
-        // the symlink may be watched before the original path.
-        // However, it stops us trying to install infinite watches when, eg,
-        // someone has created a symlink to a parent directory.
-        if( info.isSymLink() && info.exists() ){
-            if( watchingPath( info.symLinkTarget() ) )
-                return false;
-        }
         if( !Nepomuk2::FileIndexerConfig::self()->shouldFolderBeWatched( path ) )
             return false;
         // Only watch the index folders for file creation and change.
@@ -255,23 +244,7 @@ void Nepomuk2::FileWatch::slotFileCreated( const QString& path, bool isDir )
 void Nepomuk2::FileWatch::slotFileClosedAfterWrite( const QString& path )
 {
     if(FileIndexerConfig::self()->shouldBeIndexed(path)) {
-        //
-        // Check the modification time cause loads of applications open files with the write flags
-        // and do not actually modify anything
-        //
-        QFileInfo info(path);
-        QString query = QString::fromLatin1("ask where { ?r %1 %2 ; %3 %4 . }")
-                        .arg( Soprano::Node::resourceToN3(NIE::url()),
-                              Soprano::Node::resourceToN3(QUrl::fromLocalFile(path)),
-                              Soprano::Node::resourceToN3(NIE::lastModified()),
-                              Soprano::Node::literalToN3(info.lastModified().toUTC()) );
-
-        Soprano::Model* model = ResourceManager::instance()->mainModel();
-        if( !model->executeQuery(query, Soprano::Query::QueryLanguageSparql).boolValue() ) {
-            // we do not tell the file indexer right away but wait a short while in case the
-            // file is modified very often (irc logs for example)
-            m_fileModificationQueue->enqueueUrl( path );
-        }
+        m_fileModificationQueue->enqueueUrl( path );
     }
 }
 
@@ -401,8 +374,26 @@ void Nepomuk2::FileWatch::slotDeviceMounted(const Nepomuk2::RemovableMediaCache:
         kDebug() << "Device configured to not be indexed.";
     }
 
-    kDebug() << "Installing watch for removable storage at mount point" << entry->mountPath();
-    watchFolder(entry->mountPath());
+    //
+    // Install the watches
+    //
+    KConfig config("nepomukstrigirc");
+    KConfigGroup cfg = config.group( "RemovableMedia" );
+
+    if( cfg.readEntry<bool>( "add watches", true ) ) {
+        if( entry->device().isDeviceInterface( Solid::DeviceInterface::NetworkShare ) ) {
+            if( cfg.readEntry<bool>( "add watches network share", false ) ) {
+                kDebug() << "Installing watch for network share at mount point" << entry->mountPath();
+                watchFolder(entry->mountPath());
+            }
+        }
+        else {
+            kDebug() << "Installing watch for removable storage at mount point" << entry->mountPath();
+            // vHanda: Perhaps this should only be done if we have some metadata on the removable media
+            // and if we do not then we add the watches when we get some metadata?
+            watchFolder(entry->mountPath());
+        }
+    }
 }
 
 void Nepomuk2::FileWatch::slotDeviceTeardownRequested(const Nepomuk2::RemovableMediaCache::Entry* entry )
